<?php

declare(strict_types=1);

namespace FiloBlu\GcpLogging\Logger\Handler;

use Exception;
use FiloBlu\GcpLogging\Helper\Data;
use Google\Cloud\Logging\Logger;
use Google\Cloud\Logging\LoggingClient;
use Monolog\Handler\AbstractProcessingHandler;
use Psr\Log\LoggerInterface;

/**
 *
 */
class Gcp extends AbstractProcessingHandler
{
    /**
     * @var LoggerInterface[]
     */
    protected $loggers = [];

    /**
     * @var Data
     */
    protected $data;

    /**
     * Handler constructor.
     *
     * @param Data $data
     * @param int|string $level
     * @param bool $bubble
     */
    public function __construct(Data $data, $level = Logger::DEBUG, $bubble = true)
    {
        $this->data = $data;
        $this->level = $level;
        $this->bubble = $bubble;
        parent::__construct($level, $bubble);
    }

    /**
     * Writes the record down to the log of the implementing handler
     *
     * @param array $record
     * @return void
     */
    protected function write(array $record)
    {
        if (!$this->data->gcpLoggingIsEnabled() || !$this->isHandling($record)) {
            return;
        }

        try {
            $this->getLogger($record['channel'])->log(strtolower($record['level_name']), $record['message'], $record['context']);
        } catch (Exception $e) {

        }
    }

    /**
     * @param $channel
     *
     * @return LoggerInterface
     */
    protected function getLogger($channel)
    {
        $client = new LoggingClient($this->data->getLoggingClientConfiguration());

        if (!isset($this->loggers[$channel])) {
            $this->loggers[$channel] = $client->psrLogger(
                $this->data->getLogFileName(),
                ['labels' => ['context' => $channel]]
            );
        }

        return $this->loggers[$channel];
    }
}
