<?php

declare(strict_types=1);

namespace FiloBlu\GcpLogging\Logger;

use FiloBlu\GcpLogging\Helper\Data;

/**
 *
 */
class Logger extends \Monolog\Logger
{
    /**
     * @var Data
     */
    private $data;

    /**
     * @var array
     */
    private $handlerList;

    /**
     * @param \FiloBlu\GcpLogging\Helper\Data $data
     * @param string $name
     * @param array $handlers
     * @param array $processors
     */
    public function __construct(
        Data $data,
        string $name,
        array $handlers = [],
        array $processors = []

    ) {
        parent::__construct($name, $handlers, $processors);
        $this->handlerList = $handlers;
        $this->data = $data;
    }

    /**
     * Adds a log record.
     *
     * @param int $level The logging level
     * @param string $message The log message
     * @param array $context The log context
     * @return bool Whether the record has been processed
     */
    public function addRecord($level, $message, array $context = []): bool
    {
        $logType = $this->data->getGcpLoggingType();

        $handlers = [];

        foreach ($this->handlerList as $key => $instance) {
            if ($key !== $logType) {
                continue;
            }
            $handlers[$key] = $instance;
        }

        $this->setHandlers($handlers);

        $contextData = ['client_log' => $context];

        $contextData['module'] = $this->data->moduleName();
        $contextData['site'] = $this->data->getBackEndUrl();
        $contextData['stack_trace'] = debug_backtrace(DEBUG_BACKTRACE_IGNORE_ARGS, 20);

        $context = $contextData;

        return parent::addRecord($level, $message, $context);
    }
}
